# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot

snapshots = Snapshot()

snapshots["test_query_html config"] = {
    "allModelFields": {
        "auth.Group": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
            },
            "sortedFields": ["id", "admin", "name"],
        },
        "auth.User": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "date_joined": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": "datetime",
                    "prettyName": "Date joined",
                    "type": "datetime",
                },
                "email": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Email address",
                    "type": "string",
                },
                "first_name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "First name",
                    "type": "string",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "is_active": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "boolean",
                    "prettyName": "Active",
                    "type": "boolean",
                },
                "is_staff": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "boolean",
                    "prettyName": "Staff status",
                    "type": "boolean",
                },
                "is_superuser": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "boolean",
                    "prettyName": "Superuser status",
                    "type": "boolean",
                },
                "last_login": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": "datetime",
                    "prettyName": "Last login",
                    "type": "datetime",
                },
                "last_name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Last name",
                    "type": "string",
                },
                "password": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Password",
                    "type": "string",
                },
                "username": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Username",
                    "type": "string",
                },
            },
            "sortedFields": [
                "id",
                "admin",
                "date_joined",
                "email",
                "first_name",
                "is_active",
                "is_staff",
                "is_superuser",
                "last_login",
                "last_name",
                "password",
                "username",
            ],
        },
        "boolean": {
            "defaultFilters": [],
            "fields": {
                "average": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "average",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "sum": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "sum",
                    "type": "number",
                },
            },
            "sortedFields": ["average", "is_null", "sum"],
        },
        "core.Address": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "andrew": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Andrew",
                    "type": "string",
                },
                "bob": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Bob",
                    "type": "html",
                },
                "city": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "City",
                    "type": "string",
                },
                "fred": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Fred",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "producer": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.Producer",
                    "prettyName": "Producer",
                    "type": None,
                },
                "tom": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Tom",
                    "type": "html",
                },
            },
            "sortedFields": [
                "id",
                "admin",
                "andrew",
                "bob",
                "city",
                "fred",
                "producer",
                "tom",
            ],
        },
        "core.InAdmin": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
            },
            "sortedFields": ["id", "admin", "name"],
        },
        "core.InlineAdmin": {
            "defaultFilters": [],
            "fields": {
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "in_admin": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.InAdmin",
                    "prettyName": "In admin",
                    "type": None,
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
            },
            "sortedFields": ["id", "in_admin", "name"],
        },
        "core.Normal": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "in_admin": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.InAdmin",
                    "prettyName": "In admin",
                    "type": None,
                },
                "inline_admin": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.InlineAdmin",
                    "prettyName": "Inline admin",
                    "type": None,
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
            },
            "sortedFields": ["id", "admin", "in_admin", "inline_admin", "name"],
        },
        "core.Producer": {
            "defaultFilters": [],
            "fields": {
                "address": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.Address",
                    "prettyName": "Address",
                    "type": None,
                },
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "frank": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Frank",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
            },
            "sortedFields": ["id", "admin", "address", "frank", "name"],
        },
        "core.Product": {
            "defaultFilters": [
                {"lookup": "not_equals", "pathStr": "name", "value": "not a thing"}
            ],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "annotated": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Annotated",
                    "type": "string",
                },
                "boat": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "Boat",
                    "type": "number",
                },
                "calculated_boolean": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Calculated boolean",
                    "type": "boolean",
                },
                "created_time": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": "datetime",
                    "prettyName": "Created time",
                    "type": "datetime",
                },
                "date": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": "date",
                    "prettyName": "Date",
                    "type": "date",
                },
                "default_sku": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.SKU",
                    "prettyName": "Default sku",
                    "type": None,
                },
                "duration": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "duration",
                    "prettyName": "Duration",
                    "type": "duration",
                },
                "extra_inline": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Extra inline",
                    "type": "string",
                },
                "extra_model": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Extra model",
                    "type": "string",
                },
                "fake": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "unknown",
                    "prettyName": "Fake",
                    "type": "unknown",
                },
                "funky": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Funky",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "image": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "html",
                    "prettyName": "Image",
                    "type": "html",
                },
                "is_onsale": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Is onsale",
                    "type": "html",
                },
                "lambda": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Lambda",
                    "type": "html",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
                "number_choice": {
                    "canPivot": True,
                    "choices": ["A", "B"],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "numberchoice",
                    "prettyName": "Number choice",
                    "type": "numberchoice",
                },
                "only_in_list_view": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Only in list view",
                    "type": "string",
                },
                "onsale": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "boolean",
                    "prettyName": "Onsale",
                    "type": "boolean",
                },
                "producer": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.Producer",
                    "prettyName": "Producer",
                    "type": None,
                },
                "size": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "Size",
                    "type": "number",
                },
                "size_unit": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Size unit",
                    "type": "string",
                },
                "stealth_annotation": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Stealth annotation",
                    "type": "string",
                },
                "string_choice": {
                    "canPivot": True,
                    "choices": ["A", "B"],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "stringchoice",
                    "prettyName": "String choice",
                    "type": "stringchoice",
                },
            },
            "sortedFields": [
                "id",
                "admin",
                "annotated",
                "boat",
                "calculated_boolean",
                "created_time",
                "date",
                "default_sku",
                "duration",
                "extra_inline",
                "extra_model",
                "fake",
                "funky",
                "image",
                "is_onsale",
                "lambda",
                "name",
                "number_choice",
                "only_in_list_view",
                "onsale",
                "producer",
                "size",
                "size_unit",
                "stealth_annotation",
                "string_choice",
            ],
        },
        "core.SKU": {
            "defaultFilters": [],
            "fields": {
                "bob": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Bob",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
                "product": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.Product",
                    "prettyName": "Product",
                    "type": None,
                },
            },
            "sortedFields": ["id", "bob", "name", "product"],
        },
        "core.Tag": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
            },
            "sortedFields": ["id", "admin", "name"],
        },
        "data_browser.View": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "created_time": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": "datetime",
                    "prettyName": "Created time",
                    "type": "datetime",
                },
                "description": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Description",
                    "type": "string",
                },
                "fields": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Fields",
                    "type": "string",
                },
                "google_sheets_formula": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Google sheets formula",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Id",
                    "type": "string",
                },
                "limit": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "Limit",
                    "type": "number",
                },
                "model_name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Model name",
                    "type": "string",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
                "open_view": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Open view",
                    "type": "html",
                },
                "owner": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "auth.User",
                    "prettyName": "Owner",
                    "type": None,
                },
                "public": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "boolean",
                    "prettyName": "Public",
                    "type": "boolean",
                },
                "public_link": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Public link",
                    "type": "html",
                },
                "public_slug": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Public slug",
                    "type": "string",
                },
                "query": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Query",
                    "type": "string",
                },
            },
            "sortedFields": [
                "id",
                "admin",
                "created_time",
                "description",
                "fields",
                "google_sheets_formula",
                "limit",
                "model_name",
                "name",
                "open_view",
                "owner",
                "public",
                "public_link",
                "public_slug",
                "query",
            ],
        },
        "date": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "day": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "day",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "iso_week": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "iso week",
                    "type": "number",
                },
                "iso_year": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "iso year",
                    "type": "number",
                },
                "max": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "max",
                    "type": "date",
                },
                "min": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "min",
                    "type": "date",
                },
                "month": {
                    "canPivot": True,
                    "choices": [
                        "January",
                        "Feburary",
                        "March",
                        "April",
                        "May",
                        "June",
                        "July",
                        "August",
                        "September",
                        "October",
                        "November",
                        "December",
                    ],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "month",
                    "type": "numberchoice",
                },
                "month_start": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "month start",
                    "type": "date",
                },
                "quarter": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "quarter",
                    "type": "number",
                },
                "week_day": {
                    "canPivot": True,
                    "choices": [
                        "Sunday",
                        "Monday",
                        "Tuesday",
                        "Wednesday",
                        "Thursday",
                        "Friday",
                        "Saturday",
                    ],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "week day",
                    "type": "numberchoice",
                },
                "week_start": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "week start",
                    "type": "date",
                },
                "year": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "year",
                    "type": "number",
                },
            },
            "sortedFields": [
                "count",
                "day",
                "is_null",
                "iso_week",
                "iso_year",
                "max",
                "min",
                "month",
                "month_start",
                "quarter",
                "week_day",
                "week_start",
                "year",
            ],
        },
        "datetime": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "date": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "date",
                    "type": "date",
                },
                "day": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "day",
                    "type": "number",
                },
                "hour": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "hour",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "iso_week": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "iso week",
                    "type": "number",
                },
                "iso_year": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "iso year",
                    "type": "number",
                },
                "max": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "max",
                    "type": "datetime",
                },
                "min": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "min",
                    "type": "datetime",
                },
                "minute": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "minute",
                    "type": "number",
                },
                "month": {
                    "canPivot": True,
                    "choices": [
                        "January",
                        "Feburary",
                        "March",
                        "April",
                        "May",
                        "June",
                        "July",
                        "August",
                        "September",
                        "October",
                        "November",
                        "December",
                    ],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "month",
                    "type": "numberchoice",
                },
                "month_start": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "month start",
                    "type": "date",
                },
                "quarter": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "quarter",
                    "type": "number",
                },
                "second": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "second",
                    "type": "number",
                },
                "week_day": {
                    "canPivot": True,
                    "choices": [
                        "Sunday",
                        "Monday",
                        "Tuesday",
                        "Wednesday",
                        "Thursday",
                        "Friday",
                        "Saturday",
                    ],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "week day",
                    "type": "numberchoice",
                },
                "week_start": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "week start",
                    "type": "date",
                },
                "year": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "year",
                    "type": "number",
                },
            },
            "sortedFields": [
                "count",
                "date",
                "day",
                "hour",
                "is_null",
                "iso_week",
                "iso_year",
                "max",
                "min",
                "minute",
                "month",
                "month_start",
                "quarter",
                "second",
                "week_day",
                "week_start",
                "year",
            ],
        },
        "duration": {
            "defaultFilters": [],
            "fields": {
                "average": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "average",
                    "type": "duration",
                },
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "max": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "max",
                    "type": "duration",
                },
                "min": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "min",
                    "type": "duration",
                },
                "sum": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "sum",
                    "type": "duration",
                },
            },
            "sortedFields": ["average", "count", "is_null", "max", "min", "sum"],
        },
        "html": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "isnull": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "json": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "jsonfield": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "number": {
            "defaultFilters": [],
            "fields": {
                "average": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "average",
                    "type": "number",
                },
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "max": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "max",
                    "type": "number",
                },
                "min": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "min",
                    "type": "number",
                },
                "std_dev": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "std dev",
                    "type": "number",
                },
                "sum": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "sum",
                    "type": "number",
                },
                "variance": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "variance",
                    "type": "number",
                },
            },
            "sortedFields": [
                "average",
                "count",
                "is_null",
                "max",
                "min",
                "std_dev",
                "sum",
                "variance",
            ],
        },
        "numberarray": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "numberchoice": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "raw": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "raw",
                    "type": "number",
                },
            },
            "sortedFields": ["count", "is_null", "raw"],
        },
        "numberchoicearray": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "raw": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "numberarray",
                    "prettyName": "raw",
                    "type": "numberarray",
                },
            },
            "sortedFields": ["count", "is_null", "raw"],
        },
        "regex": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "string": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "length": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "length",
                    "type": "number",
                },
            },
            "sortedFields": ["count", "is_null", "length"],
        },
        "stringarray": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "stringchoice": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "raw": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "raw",
                    "type": "string",
                },
            },
            "sortedFields": ["count", "is_null", "raw"],
        },
        "stringchoicearray": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "raw": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "stringarray",
                    "prettyName": "raw",
                    "type": "stringarray",
                },
            },
            "sortedFields": ["count", "is_null", "raw"],
        },
        "unknown": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
    },
    "baseUrl": "/data_browser/",
    "canMakePublic": True,
    "defaultRowLimit": 1000,
    "sentryDsn": None,
    "sortedModels": [
        "auth.Group",
        "auth.User",
        "core.Address",
        "core.InAdmin",
        "core.InlineAdmin",
        "core.Normal",
        "core.Producer",
        "core.Product",
        "core.SKU",
        "core.Tag",
        "data_browser.View",
    ],
    "types": {
        "boolean": {
            "defaultLookup": "equals",
            "defaultValue": True,
            "lookups": {
                "equals": {"type": "boolean"},
                "is_null": {"type": "isnull"},
                "not_equals": {"type": "boolean"},
            },
            "sortedLookups": ["equals", "not_equals", "is_null"],
        },
        "date": {
            "defaultLookup": "equals",
            "defaultValue": "today",
            "lookups": {
                "equals": {"type": "date"},
                "gt": {"type": "date"},
                "gte": {"type": "date"},
                "is_null": {"type": "isnull"},
                "lt": {"type": "date"},
                "lte": {"type": "date"},
                "not_equals": {"type": "date"},
            },
            "sortedLookups": [
                "equals",
                "not_equals",
                "gt",
                "gte",
                "lt",
                "lte",
                "is_null",
            ],
        },
        "datetime": {
            "defaultLookup": "equals",
            "defaultValue": "now",
            "lookups": {
                "equals": {"type": "datetime"},
                "gt": {"type": "datetime"},
                "gte": {"type": "datetime"},
                "is_null": {"type": "isnull"},
                "lt": {"type": "datetime"},
                "lte": {"type": "datetime"},
                "not_equals": {"type": "datetime"},
            },
            "sortedLookups": [
                "equals",
                "not_equals",
                "gt",
                "gte",
                "lt",
                "lte",
                "is_null",
            ],
        },
        "duration": {
            "defaultLookup": "equals",
            "defaultValue": "",
            "lookups": {
                "equals": {"type": "duration"},
                "gt": {"type": "duration"},
                "gte": {"type": "duration"},
                "is_null": {"type": "isnull"},
                "lt": {"type": "duration"},
                "lte": {"type": "duration"},
                "not_equals": {"type": "duration"},
            },
            "sortedLookups": [
                "equals",
                "not_equals",
                "gt",
                "gte",
                "lt",
                "lte",
                "is_null",
            ],
        },
        "html": {
            "defaultLookup": None,
            "defaultValue": "",
            "lookups": {},
            "sortedLookups": [],
        },
        "isnull": {
            "defaultLookup": "equals",
            "defaultValue": "IsNull",
            "lookups": {"equals": {"type": "isnull"}},
            "sortedLookups": ["equals"],
        },
        "json": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "equals": {"type": "json"},
                "field_equals": {"type": "jsonfield"},
                "has_key": {"type": "string"},
                "is_null": {"type": "isnull"},
                "not_equals": {"type": "json"},
                "not_field_equals": {"type": "jsonfield"},
                "not_has_key": {"type": "string"},
            },
            "sortedLookups": [
                "equals",
                "has_key",
                "field_equals",
                "not_equals",
                "not_has_key",
                "not_field_equals",
                "is_null",
            ],
        },
        "jsonfield": {
            "defaultLookup": None,
            "defaultValue": "|",
            "lookups": {},
            "sortedLookups": [],
        },
        "number": {
            "defaultLookup": "equals",
            "defaultValue": 0,
            "lookups": {
                "equals": {"type": "number"},
                "gt": {"type": "number"},
                "gte": {"type": "number"},
                "is_null": {"type": "isnull"},
                "lt": {"type": "number"},
                "lte": {"type": "number"},
                "not_equals": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "not_equals",
                "gt",
                "gte",
                "lt",
                "lte",
                "is_null",
            ],
        },
        "numberarray": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "contains": {"type": "number"},
                "equals": {"type": "numberarray"},
                "is_null": {"type": "isnull"},
                "length": {"type": "number"},
                "not_contains": {"type": "number"},
                "not_equals": {"type": "numberarray"},
                "not_length": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "length",
                "not_equals",
                "not_contains",
                "not_length",
                "is_null",
            ],
        },
        "numberchoice": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "equals": {"type": "numberchoice"},
                "is_null": {"type": "isnull"},
                "not_equals": {"type": "numberchoice"},
            },
            "sortedLookups": ["equals", "not_equals", "is_null"],
        },
        "numberchoicearray": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "contains": {"type": "numberchoice"},
                "equals": {"type": "numberchoicearray"},
                "is_null": {"type": "isnull"},
                "length": {"type": "number"},
                "not_contains": {"type": "numberchoice"},
                "not_equals": {"type": "numberchoicearray"},
                "not_length": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "length",
                "not_equals",
                "not_contains",
                "not_length",
                "is_null",
            ],
        },
        "regex": {
            "defaultLookup": None,
            "defaultValue": ".*",
            "lookups": {},
            "sortedLookups": [],
        },
        "string": {
            "defaultLookup": "equals",
            "defaultValue": "",
            "lookups": {
                "contains": {"type": "string"},
                "ends_with": {"type": "string"},
                "equals": {"type": "string"},
                "is_null": {"type": "isnull"},
                "not_contains": {"type": "string"},
                "not_ends_with": {"type": "string"},
                "not_equals": {"type": "string"},
                "not_regex": {"type": "regex"},
                "not_starts_with": {"type": "string"},
                "regex": {"type": "regex"},
                "starts_with": {"type": "string"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "starts_with",
                "ends_with",
                "regex",
                "not_equals",
                "not_contains",
                "not_starts_with",
                "not_ends_with",
                "not_regex",
                "is_null",
            ],
        },
        "stringarray": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "contains": {"type": "string"},
                "equals": {"type": "stringarray"},
                "is_null": {"type": "isnull"},
                "length": {"type": "number"},
                "not_contains": {"type": "string"},
                "not_equals": {"type": "stringarray"},
                "not_length": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "length",
                "not_equals",
                "not_contains",
                "not_length",
                "is_null",
            ],
        },
        "stringchoice": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "equals": {"type": "stringchoice"},
                "is_null": {"type": "isnull"},
                "not_equals": {"type": "stringchoice"},
            },
            "sortedLookups": ["equals", "not_equals", "is_null"],
        },
        "stringchoicearray": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "contains": {"type": "stringchoice"},
                "equals": {"type": "stringchoicearray"},
                "is_null": {"type": "isnull"},
                "length": {"type": "number"},
                "not_contains": {"type": "stringchoice"},
                "not_equals": {"type": "stringchoicearray"},
                "not_length": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "length",
                "not_equals",
                "not_contains",
                "not_length",
                "is_null",
            ],
        },
        "unknown": {
            "defaultLookup": "is_null",
            "defaultValue": None,
            "lookups": {"is_null": {"type": "isnull"}},
            "sortedLookups": ["is_null"],
        },
    },
}

snapshots["test_query_query query"] = {
    "fields": [
        {"pathStr": "size", "pivoted": False, "priority": 0, "sort": "dsc"},
        {"pathStr": "name", "pivoted": False, "priority": 1, "sort": "asc"},
        {"pathStr": "size_unit", "pivoted": False, "priority": None, "sort": None},
    ],
    "filterErrors": [None, None],
    "filters": [
        {"lookup": "lt", "pathStr": "size", "value": "2"},
        {"lookup": "gt", "pathStr": "id", "value": "0"},
    ],
    "limit": 1000,
    "model": "core.Product",
}

snapshots["test_query_html_no_perms config"] = {
    "allModelFields": {
        "boolean": {
            "defaultFilters": [],
            "fields": {
                "average": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "average",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "sum": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "sum",
                    "type": "number",
                },
            },
            "sortedFields": ["average", "is_null", "sum"],
        },
        "date": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "day": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "day",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "iso_week": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "iso week",
                    "type": "number",
                },
                "iso_year": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "iso year",
                    "type": "number",
                },
                "max": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "max",
                    "type": "date",
                },
                "min": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "min",
                    "type": "date",
                },
                "month": {
                    "canPivot": True,
                    "choices": [
                        "January",
                        "Feburary",
                        "March",
                        "April",
                        "May",
                        "June",
                        "July",
                        "August",
                        "September",
                        "October",
                        "November",
                        "December",
                    ],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "month",
                    "type": "numberchoice",
                },
                "month_start": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "month start",
                    "type": "date",
                },
                "quarter": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "quarter",
                    "type": "number",
                },
                "week_day": {
                    "canPivot": True,
                    "choices": [
                        "Sunday",
                        "Monday",
                        "Tuesday",
                        "Wednesday",
                        "Thursday",
                        "Friday",
                        "Saturday",
                    ],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "week day",
                    "type": "numberchoice",
                },
                "week_start": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "week start",
                    "type": "date",
                },
                "year": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "year",
                    "type": "number",
                },
            },
            "sortedFields": [
                "count",
                "day",
                "is_null",
                "iso_week",
                "iso_year",
                "max",
                "min",
                "month",
                "month_start",
                "quarter",
                "week_day",
                "week_start",
                "year",
            ],
        },
        "datetime": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "date": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "date",
                    "type": "date",
                },
                "day": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "day",
                    "type": "number",
                },
                "hour": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "hour",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "iso_week": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "iso week",
                    "type": "number",
                },
                "iso_year": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "iso year",
                    "type": "number",
                },
                "max": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "max",
                    "type": "datetime",
                },
                "min": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "min",
                    "type": "datetime",
                },
                "minute": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "minute",
                    "type": "number",
                },
                "month": {
                    "canPivot": True,
                    "choices": [
                        "January",
                        "Feburary",
                        "March",
                        "April",
                        "May",
                        "June",
                        "July",
                        "August",
                        "September",
                        "October",
                        "November",
                        "December",
                    ],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "month",
                    "type": "numberchoice",
                },
                "month_start": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "month start",
                    "type": "date",
                },
                "quarter": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "quarter",
                    "type": "number",
                },
                "second": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "second",
                    "type": "number",
                },
                "week_day": {
                    "canPivot": True,
                    "choices": [
                        "Sunday",
                        "Monday",
                        "Tuesday",
                        "Wednesday",
                        "Thursday",
                        "Friday",
                        "Saturday",
                    ],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "week day",
                    "type": "numberchoice",
                },
                "week_start": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "week start",
                    "type": "date",
                },
                "year": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "year",
                    "type": "number",
                },
            },
            "sortedFields": [
                "count",
                "date",
                "day",
                "hour",
                "is_null",
                "iso_week",
                "iso_year",
                "max",
                "min",
                "minute",
                "month",
                "month_start",
                "quarter",
                "second",
                "week_day",
                "week_start",
                "year",
            ],
        },
        "duration": {
            "defaultFilters": [],
            "fields": {
                "average": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "average",
                    "type": "duration",
                },
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "max": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "max",
                    "type": "duration",
                },
                "min": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "min",
                    "type": "duration",
                },
                "sum": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "sum",
                    "type": "duration",
                },
            },
            "sortedFields": ["average", "count", "is_null", "max", "min", "sum"],
        },
        "html": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "isnull": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "json": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "jsonfield": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "number": {
            "defaultFilters": [],
            "fields": {
                "average": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "average",
                    "type": "number",
                },
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "max": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "max",
                    "type": "number",
                },
                "min": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "min",
                    "type": "number",
                },
                "std_dev": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "std dev",
                    "type": "number",
                },
                "sum": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "sum",
                    "type": "number",
                },
                "variance": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "variance",
                    "type": "number",
                },
            },
            "sortedFields": [
                "average",
                "count",
                "is_null",
                "max",
                "min",
                "std_dev",
                "sum",
                "variance",
            ],
        },
        "numberarray": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "numberchoice": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "raw": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "raw",
                    "type": "number",
                },
            },
            "sortedFields": ["count", "is_null", "raw"],
        },
        "numberchoicearray": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "raw": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "numberarray",
                    "prettyName": "raw",
                    "type": "numberarray",
                },
            },
            "sortedFields": ["count", "is_null", "raw"],
        },
        "regex": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "string": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "length": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "length",
                    "type": "number",
                },
            },
            "sortedFields": ["count", "is_null", "length"],
        },
        "stringarray": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "stringchoice": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "raw": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "raw",
                    "type": "string",
                },
            },
            "sortedFields": ["count", "is_null", "raw"],
        },
        "stringchoicearray": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "raw": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "stringarray",
                    "prettyName": "raw",
                    "type": "stringarray",
                },
            },
            "sortedFields": ["count", "is_null", "raw"],
        },
        "unknown": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
    },
    "baseUrl": "/data_browser/",
    "canMakePublic": False,
    "defaultRowLimit": 1000,
    "sentryDsn": None,
    "sortedModels": [],
    "types": {
        "boolean": {
            "defaultLookup": "equals",
            "defaultValue": True,
            "lookups": {
                "equals": {"type": "boolean"},
                "is_null": {"type": "isnull"},
                "not_equals": {"type": "boolean"},
            },
            "sortedLookups": ["equals", "not_equals", "is_null"],
        },
        "date": {
            "defaultLookup": "equals",
            "defaultValue": "today",
            "lookups": {
                "equals": {"type": "date"},
                "gt": {"type": "date"},
                "gte": {"type": "date"},
                "is_null": {"type": "isnull"},
                "lt": {"type": "date"},
                "lte": {"type": "date"},
                "not_equals": {"type": "date"},
            },
            "sortedLookups": [
                "equals",
                "not_equals",
                "gt",
                "gte",
                "lt",
                "lte",
                "is_null",
            ],
        },
        "datetime": {
            "defaultLookup": "equals",
            "defaultValue": "now",
            "lookups": {
                "equals": {"type": "datetime"},
                "gt": {"type": "datetime"},
                "gte": {"type": "datetime"},
                "is_null": {"type": "isnull"},
                "lt": {"type": "datetime"},
                "lte": {"type": "datetime"},
                "not_equals": {"type": "datetime"},
            },
            "sortedLookups": [
                "equals",
                "not_equals",
                "gt",
                "gte",
                "lt",
                "lte",
                "is_null",
            ],
        },
        "duration": {
            "defaultLookup": "equals",
            "defaultValue": "",
            "lookups": {
                "equals": {"type": "duration"},
                "gt": {"type": "duration"},
                "gte": {"type": "duration"},
                "is_null": {"type": "isnull"},
                "lt": {"type": "duration"},
                "lte": {"type": "duration"},
                "not_equals": {"type": "duration"},
            },
            "sortedLookups": [
                "equals",
                "not_equals",
                "gt",
                "gte",
                "lt",
                "lte",
                "is_null",
            ],
        },
        "html": {
            "defaultLookup": None,
            "defaultValue": "",
            "lookups": {},
            "sortedLookups": [],
        },
        "isnull": {
            "defaultLookup": "equals",
            "defaultValue": "IsNull",
            "lookups": {"equals": {"type": "isnull"}},
            "sortedLookups": ["equals"],
        },
        "json": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "equals": {"type": "json"},
                "field_equals": {"type": "jsonfield"},
                "has_key": {"type": "string"},
                "is_null": {"type": "isnull"},
                "not_equals": {"type": "json"},
                "not_field_equals": {"type": "jsonfield"},
                "not_has_key": {"type": "string"},
            },
            "sortedLookups": [
                "equals",
                "has_key",
                "field_equals",
                "not_equals",
                "not_has_key",
                "not_field_equals",
                "is_null",
            ],
        },
        "jsonfield": {
            "defaultLookup": None,
            "defaultValue": "|",
            "lookups": {},
            "sortedLookups": [],
        },
        "number": {
            "defaultLookup": "equals",
            "defaultValue": 0,
            "lookups": {
                "equals": {"type": "number"},
                "gt": {"type": "number"},
                "gte": {"type": "number"},
                "is_null": {"type": "isnull"},
                "lt": {"type": "number"},
                "lte": {"type": "number"},
                "not_equals": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "not_equals",
                "gt",
                "gte",
                "lt",
                "lte",
                "is_null",
            ],
        },
        "numberarray": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "contains": {"type": "number"},
                "equals": {"type": "numberarray"},
                "is_null": {"type": "isnull"},
                "length": {"type": "number"},
                "not_contains": {"type": "number"},
                "not_equals": {"type": "numberarray"},
                "not_length": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "length",
                "not_equals",
                "not_contains",
                "not_length",
                "is_null",
            ],
        },
        "numberchoice": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "equals": {"type": "numberchoice"},
                "is_null": {"type": "isnull"},
                "not_equals": {"type": "numberchoice"},
            },
            "sortedLookups": ["equals", "not_equals", "is_null"],
        },
        "numberchoicearray": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "contains": {"type": "numberchoice"},
                "equals": {"type": "numberchoicearray"},
                "is_null": {"type": "isnull"},
                "length": {"type": "number"},
                "not_contains": {"type": "numberchoice"},
                "not_equals": {"type": "numberchoicearray"},
                "not_length": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "length",
                "not_equals",
                "not_contains",
                "not_length",
                "is_null",
            ],
        },
        "regex": {
            "defaultLookup": None,
            "defaultValue": ".*",
            "lookups": {},
            "sortedLookups": [],
        },
        "string": {
            "defaultLookup": "equals",
            "defaultValue": "",
            "lookups": {
                "contains": {"type": "string"},
                "ends_with": {"type": "string"},
                "equals": {"type": "string"},
                "is_null": {"type": "isnull"},
                "not_contains": {"type": "string"},
                "not_ends_with": {"type": "string"},
                "not_equals": {"type": "string"},
                "not_regex": {"type": "regex"},
                "not_starts_with": {"type": "string"},
                "regex": {"type": "regex"},
                "starts_with": {"type": "string"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "starts_with",
                "ends_with",
                "regex",
                "not_equals",
                "not_contains",
                "not_starts_with",
                "not_ends_with",
                "not_regex",
                "is_null",
            ],
        },
        "stringarray": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "contains": {"type": "string"},
                "equals": {"type": "stringarray"},
                "is_null": {"type": "isnull"},
                "length": {"type": "number"},
                "not_contains": {"type": "string"},
                "not_equals": {"type": "stringarray"},
                "not_length": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "length",
                "not_equals",
                "not_contains",
                "not_length",
                "is_null",
            ],
        },
        "stringchoice": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "equals": {"type": "stringchoice"},
                "is_null": {"type": "isnull"},
                "not_equals": {"type": "stringchoice"},
            },
            "sortedLookups": ["equals", "not_equals", "is_null"],
        },
        "stringchoicearray": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "contains": {"type": "stringchoice"},
                "equals": {"type": "stringchoicearray"},
                "is_null": {"type": "isnull"},
                "length": {"type": "number"},
                "not_contains": {"type": "stringchoice"},
                "not_equals": {"type": "stringchoicearray"},
                "not_length": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "length",
                "not_equals",
                "not_contains",
                "not_length",
                "is_null",
            ],
        },
        "unknown": {
            "defaultLookup": "is_null",
            "defaultValue": None,
            "lookups": {"is_null": {"type": "isnull"}},
            "sortedLookups": ["is_null"],
        },
    },
}

snapshots["test_query_ctx config"] = {
    "allModelFields": {
        "auth.Group": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
            },
            "sortedFields": ["id", "admin", "name"],
        },
        "auth.User": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "date_joined": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": "datetime",
                    "prettyName": "Date joined",
                    "type": "datetime",
                },
                "email": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Email address",
                    "type": "string",
                },
                "first_name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "First name",
                    "type": "string",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "is_active": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "boolean",
                    "prettyName": "Active",
                    "type": "boolean",
                },
                "is_staff": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "boolean",
                    "prettyName": "Staff status",
                    "type": "boolean",
                },
                "is_superuser": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "boolean",
                    "prettyName": "Superuser status",
                    "type": "boolean",
                },
                "last_login": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": "datetime",
                    "prettyName": "Last login",
                    "type": "datetime",
                },
                "last_name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Last name",
                    "type": "string",
                },
                "password": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Password",
                    "type": "string",
                },
                "username": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Username",
                    "type": "string",
                },
            },
            "sortedFields": [
                "id",
                "admin",
                "date_joined",
                "email",
                "first_name",
                "is_active",
                "is_staff",
                "is_superuser",
                "last_login",
                "last_name",
                "password",
                "username",
            ],
        },
        "boolean": {
            "defaultFilters": [],
            "fields": {
                "average": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "average",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "sum": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "sum",
                    "type": "number",
                },
            },
            "sortedFields": ["average", "is_null", "sum"],
        },
        "core.Address": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "andrew": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Andrew",
                    "type": "string",
                },
                "bob": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Bob",
                    "type": "html",
                },
                "city": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "City",
                    "type": "string",
                },
                "fred": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Fred",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "producer": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.Producer",
                    "prettyName": "Producer",
                    "type": None,
                },
                "tom": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Tom",
                    "type": "html",
                },
            },
            "sortedFields": [
                "id",
                "admin",
                "andrew",
                "bob",
                "city",
                "fred",
                "producer",
                "tom",
            ],
        },
        "core.InAdmin": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
            },
            "sortedFields": ["id", "admin", "name"],
        },
        "core.InlineAdmin": {
            "defaultFilters": [],
            "fields": {
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "in_admin": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.InAdmin",
                    "prettyName": "In admin",
                    "type": None,
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
            },
            "sortedFields": ["id", "in_admin", "name"],
        },
        "core.Normal": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "in_admin": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.InAdmin",
                    "prettyName": "In admin",
                    "type": None,
                },
                "inline_admin": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.InlineAdmin",
                    "prettyName": "Inline admin",
                    "type": None,
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
            },
            "sortedFields": ["id", "admin", "in_admin", "inline_admin", "name"],
        },
        "core.Producer": {
            "defaultFilters": [],
            "fields": {
                "address": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.Address",
                    "prettyName": "Address",
                    "type": None,
                },
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "frank": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Frank",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
            },
            "sortedFields": ["id", "admin", "address", "frank", "name"],
        },
        "core.Product": {
            "defaultFilters": [
                {"lookup": "not_equals", "pathStr": "name", "value": "not a thing"}
            ],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "annotated": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Annotated",
                    "type": "string",
                },
                "boat": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "Boat",
                    "type": "number",
                },
                "calculated_boolean": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Calculated boolean",
                    "type": "boolean",
                },
                "created_time": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": "datetime",
                    "prettyName": "Created time",
                    "type": "datetime",
                },
                "date": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": "date",
                    "prettyName": "Date",
                    "type": "date",
                },
                "default_sku": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.SKU",
                    "prettyName": "Default sku",
                    "type": None,
                },
                "duration": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "duration",
                    "prettyName": "Duration",
                    "type": "duration",
                },
                "extra_inline": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Extra inline",
                    "type": "string",
                },
                "extra_model": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Extra model",
                    "type": "string",
                },
                "fake": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "unknown",
                    "prettyName": "Fake",
                    "type": "unknown",
                },
                "funky": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Funky",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "image": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "html",
                    "prettyName": "Image",
                    "type": "html",
                },
                "is_onsale": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Is onsale",
                    "type": "html",
                },
                "lambda": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Lambda",
                    "type": "html",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
                "number_choice": {
                    "canPivot": True,
                    "choices": ["A", "B"],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "numberchoice",
                    "prettyName": "Number choice",
                    "type": "numberchoice",
                },
                "only_in_list_view": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Only in list view",
                    "type": "string",
                },
                "onsale": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "boolean",
                    "prettyName": "Onsale",
                    "type": "boolean",
                },
                "producer": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.Producer",
                    "prettyName": "Producer",
                    "type": None,
                },
                "size": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "Size",
                    "type": "number",
                },
                "size_unit": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Size unit",
                    "type": "string",
                },
                "stealth_annotation": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Stealth annotation",
                    "type": "string",
                },
                "string_choice": {
                    "canPivot": True,
                    "choices": ["A", "B"],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "stringchoice",
                    "prettyName": "String choice",
                    "type": "stringchoice",
                },
            },
            "sortedFields": [
                "id",
                "admin",
                "annotated",
                "boat",
                "calculated_boolean",
                "created_time",
                "date",
                "default_sku",
                "duration",
                "extra_inline",
                "extra_model",
                "fake",
                "funky",
                "image",
                "is_onsale",
                "lambda",
                "name",
                "number_choice",
                "only_in_list_view",
                "onsale",
                "producer",
                "size",
                "size_unit",
                "stealth_annotation",
                "string_choice",
            ],
        },
        "core.SKU": {
            "defaultFilters": [],
            "fields": {
                "bob": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Bob",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
                "product": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "core.Product",
                    "prettyName": "Product",
                    "type": None,
                },
            },
            "sortedFields": ["id", "bob", "name", "product"],
        },
        "core.Tag": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "ID",
                    "type": "number",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
            },
            "sortedFields": ["id", "admin", "name"],
        },
        "data_browser.View": {
            "defaultFilters": [],
            "fields": {
                "admin": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Admin",
                    "type": "html",
                },
                "created_time": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": "datetime",
                    "prettyName": "Created time",
                    "type": "datetime",
                },
                "description": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Description",
                    "type": "string",
                },
                "fields": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Fields",
                    "type": "string",
                },
                "google_sheets_formula": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Google sheets formula",
                    "type": "html",
                },
                "id": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Id",
                    "type": "string",
                },
                "limit": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "Limit",
                    "type": "number",
                },
                "model_name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Model name",
                    "type": "string",
                },
                "name": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Name",
                    "type": "string",
                },
                "open_view": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Open view",
                    "type": "html",
                },
                "owner": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": "auth.User",
                    "prettyName": "Owner",
                    "type": None,
                },
                "public": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "boolean",
                    "prettyName": "Public",
                    "type": "boolean",
                },
                "public_link": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": False,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "Public link",
                    "type": "html",
                },
                "public_slug": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Public slug",
                    "type": "string",
                },
                "query": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "Query",
                    "type": "string",
                },
            },
            "sortedFields": [
                "id",
                "admin",
                "created_time",
                "description",
                "fields",
                "google_sheets_formula",
                "limit",
                "model_name",
                "name",
                "open_view",
                "owner",
                "public",
                "public_link",
                "public_slug",
                "query",
            ],
        },
        "date": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "day": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "day",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "iso_week": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "iso week",
                    "type": "number",
                },
                "iso_year": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "iso year",
                    "type": "number",
                },
                "max": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "max",
                    "type": "date",
                },
                "min": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "min",
                    "type": "date",
                },
                "month": {
                    "canPivot": True,
                    "choices": [
                        "January",
                        "Feburary",
                        "March",
                        "April",
                        "May",
                        "June",
                        "July",
                        "August",
                        "September",
                        "October",
                        "November",
                        "December",
                    ],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "month",
                    "type": "numberchoice",
                },
                "month_start": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "month start",
                    "type": "date",
                },
                "quarter": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "quarter",
                    "type": "number",
                },
                "week_day": {
                    "canPivot": True,
                    "choices": [
                        "Sunday",
                        "Monday",
                        "Tuesday",
                        "Wednesday",
                        "Thursday",
                        "Friday",
                        "Saturday",
                    ],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "week day",
                    "type": "numberchoice",
                },
                "week_start": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "week start",
                    "type": "date",
                },
                "year": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "year",
                    "type": "number",
                },
            },
            "sortedFields": [
                "count",
                "day",
                "is_null",
                "iso_week",
                "iso_year",
                "max",
                "min",
                "month",
                "month_start",
                "quarter",
                "week_day",
                "week_start",
                "year",
            ],
        },
        "datetime": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "date": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "date",
                    "type": "date",
                },
                "day": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "day",
                    "type": "number",
                },
                "hour": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "hour",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "iso_week": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "iso week",
                    "type": "number",
                },
                "iso_year": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "iso year",
                    "type": "number",
                },
                "max": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "max",
                    "type": "datetime",
                },
                "min": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "min",
                    "type": "datetime",
                },
                "minute": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "minute",
                    "type": "number",
                },
                "month": {
                    "canPivot": True,
                    "choices": [
                        "January",
                        "Feburary",
                        "March",
                        "April",
                        "May",
                        "June",
                        "July",
                        "August",
                        "September",
                        "October",
                        "November",
                        "December",
                    ],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "month",
                    "type": "numberchoice",
                },
                "month_start": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "month start",
                    "type": "date",
                },
                "quarter": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "quarter",
                    "type": "number",
                },
                "second": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "second",
                    "type": "number",
                },
                "week_day": {
                    "canPivot": True,
                    "choices": [
                        "Sunday",
                        "Monday",
                        "Tuesday",
                        "Wednesday",
                        "Thursday",
                        "Friday",
                        "Saturday",
                    ],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "week day",
                    "type": "numberchoice",
                },
                "week_start": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "week start",
                    "type": "date",
                },
                "year": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": "asc",
                    "model": None,
                    "prettyName": "year",
                    "type": "number",
                },
            },
            "sortedFields": [
                "count",
                "date",
                "day",
                "hour",
                "is_null",
                "iso_week",
                "iso_year",
                "max",
                "min",
                "minute",
                "month",
                "month_start",
                "quarter",
                "second",
                "week_day",
                "week_start",
                "year",
            ],
        },
        "duration": {
            "defaultFilters": [],
            "fields": {
                "average": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "average",
                    "type": "duration",
                },
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "max": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "max",
                    "type": "duration",
                },
                "min": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "min",
                    "type": "duration",
                },
                "sum": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "sum",
                    "type": "duration",
                },
            },
            "sortedFields": ["average", "count", "is_null", "max", "min", "sum"],
        },
        "html": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "isnull": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "json": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "jsonfield": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "number": {
            "defaultFilters": [],
            "fields": {
                "average": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "average",
                    "type": "number",
                },
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "max": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "max",
                    "type": "number",
                },
                "min": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "min",
                    "type": "number",
                },
                "std_dev": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "std dev",
                    "type": "number",
                },
                "sum": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "sum",
                    "type": "number",
                },
                "variance": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "variance",
                    "type": "number",
                },
            },
            "sortedFields": [
                "average",
                "count",
                "is_null",
                "max",
                "min",
                "std_dev",
                "sum",
                "variance",
            ],
        },
        "numberarray": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "numberchoice": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "raw": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "number",
                    "prettyName": "raw",
                    "type": "number",
                },
            },
            "sortedFields": ["count", "is_null", "raw"],
        },
        "numberchoicearray": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "raw": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "numberarray",
                    "prettyName": "raw",
                    "type": "numberarray",
                },
            },
            "sortedFields": ["count", "is_null", "raw"],
        },
        "regex": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "string": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "length": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "length",
                    "type": "number",
                },
            },
            "sortedFields": ["count", "is_null", "length"],
        },
        "stringarray": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
        "stringchoice": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "raw": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "string",
                    "prettyName": "raw",
                    "type": "string",
                },
            },
            "sortedFields": ["count", "is_null", "raw"],
        },
        "stringchoicearray": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
                "raw": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": "stringarray",
                    "prettyName": "raw",
                    "type": "stringarray",
                },
            },
            "sortedFields": ["count", "is_null", "raw"],
        },
        "unknown": {
            "defaultFilters": [],
            "fields": {
                "count": {
                    "canPivot": False,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "count",
                    "type": "number",
                },
                "is_null": {
                    "canPivot": True,
                    "choices": [],
                    "concrete": True,
                    "defaultSort": None,
                    "model": None,
                    "prettyName": "is null",
                    "type": "isnull",
                },
            },
            "sortedFields": ["count", "is_null"],
        },
    },
    "baseUrl": "/data_browser/",
    "canMakePublic": True,
    "defaultRowLimit": 1000,
    "sentryDsn": None,
    "sortedModels": [
        "auth.Group",
        "auth.User",
        "core.Address",
        "core.InAdmin",
        "core.InlineAdmin",
        "core.Normal",
        "core.Producer",
        "core.Product",
        "core.SKU",
        "core.Tag",
        "data_browser.View",
    ],
    "types": {
        "boolean": {
            "defaultLookup": "equals",
            "defaultValue": True,
            "lookups": {
                "equals": {"type": "boolean"},
                "is_null": {"type": "isnull"},
                "not_equals": {"type": "boolean"},
            },
            "sortedLookups": ["equals", "not_equals", "is_null"],
        },
        "date": {
            "defaultLookup": "equals",
            "defaultValue": "today",
            "lookups": {
                "equals": {"type": "date"},
                "gt": {"type": "date"},
                "gte": {"type": "date"},
                "is_null": {"type": "isnull"},
                "lt": {"type": "date"},
                "lte": {"type": "date"},
                "not_equals": {"type": "date"},
            },
            "sortedLookups": [
                "equals",
                "not_equals",
                "gt",
                "gte",
                "lt",
                "lte",
                "is_null",
            ],
        },
        "datetime": {
            "defaultLookup": "equals",
            "defaultValue": "now",
            "lookups": {
                "equals": {"type": "datetime"},
                "gt": {"type": "datetime"},
                "gte": {"type": "datetime"},
                "is_null": {"type": "isnull"},
                "lt": {"type": "datetime"},
                "lte": {"type": "datetime"},
                "not_equals": {"type": "datetime"},
            },
            "sortedLookups": [
                "equals",
                "not_equals",
                "gt",
                "gte",
                "lt",
                "lte",
                "is_null",
            ],
        },
        "duration": {
            "defaultLookup": "equals",
            "defaultValue": "",
            "lookups": {
                "equals": {"type": "duration"},
                "gt": {"type": "duration"},
                "gte": {"type": "duration"},
                "is_null": {"type": "isnull"},
                "lt": {"type": "duration"},
                "lte": {"type": "duration"},
                "not_equals": {"type": "duration"},
            },
            "sortedLookups": [
                "equals",
                "not_equals",
                "gt",
                "gte",
                "lt",
                "lte",
                "is_null",
            ],
        },
        "html": {
            "defaultLookup": None,
            "defaultValue": "",
            "lookups": {},
            "sortedLookups": [],
        },
        "isnull": {
            "defaultLookup": "equals",
            "defaultValue": "IsNull",
            "lookups": {"equals": {"type": "isnull"}},
            "sortedLookups": ["equals"],
        },
        "json": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "equals": {"type": "json"},
                "field_equals": {"type": "jsonfield"},
                "has_key": {"type": "string"},
                "is_null": {"type": "isnull"},
                "not_equals": {"type": "json"},
                "not_field_equals": {"type": "jsonfield"},
                "not_has_key": {"type": "string"},
            },
            "sortedLookups": [
                "equals",
                "has_key",
                "field_equals",
                "not_equals",
                "not_has_key",
                "not_field_equals",
                "is_null",
            ],
        },
        "jsonfield": {
            "defaultLookup": None,
            "defaultValue": "|",
            "lookups": {},
            "sortedLookups": [],
        },
        "number": {
            "defaultLookup": "equals",
            "defaultValue": 0,
            "lookups": {
                "equals": {"type": "number"},
                "gt": {"type": "number"},
                "gte": {"type": "number"},
                "is_null": {"type": "isnull"},
                "lt": {"type": "number"},
                "lte": {"type": "number"},
                "not_equals": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "not_equals",
                "gt",
                "gte",
                "lt",
                "lte",
                "is_null",
            ],
        },
        "numberarray": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "contains": {"type": "number"},
                "equals": {"type": "numberarray"},
                "is_null": {"type": "isnull"},
                "length": {"type": "number"},
                "not_contains": {"type": "number"},
                "not_equals": {"type": "numberarray"},
                "not_length": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "length",
                "not_equals",
                "not_contains",
                "not_length",
                "is_null",
            ],
        },
        "numberchoice": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "equals": {"type": "numberchoice"},
                "is_null": {"type": "isnull"},
                "not_equals": {"type": "numberchoice"},
            },
            "sortedLookups": ["equals", "not_equals", "is_null"],
        },
        "numberchoicearray": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "contains": {"type": "numberchoice"},
                "equals": {"type": "numberchoicearray"},
                "is_null": {"type": "isnull"},
                "length": {"type": "number"},
                "not_contains": {"type": "numberchoice"},
                "not_equals": {"type": "numberchoicearray"},
                "not_length": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "length",
                "not_equals",
                "not_contains",
                "not_length",
                "is_null",
            ],
        },
        "regex": {
            "defaultLookup": None,
            "defaultValue": ".*",
            "lookups": {},
            "sortedLookups": [],
        },
        "string": {
            "defaultLookup": "equals",
            "defaultValue": "",
            "lookups": {
                "contains": {"type": "string"},
                "ends_with": {"type": "string"},
                "equals": {"type": "string"},
                "is_null": {"type": "isnull"},
                "not_contains": {"type": "string"},
                "not_ends_with": {"type": "string"},
                "not_equals": {"type": "string"},
                "not_regex": {"type": "regex"},
                "not_starts_with": {"type": "string"},
                "regex": {"type": "regex"},
                "starts_with": {"type": "string"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "starts_with",
                "ends_with",
                "regex",
                "not_equals",
                "not_contains",
                "not_starts_with",
                "not_ends_with",
                "not_regex",
                "is_null",
            ],
        },
        "stringarray": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "contains": {"type": "string"},
                "equals": {"type": "stringarray"},
                "is_null": {"type": "isnull"},
                "length": {"type": "number"},
                "not_contains": {"type": "string"},
                "not_equals": {"type": "stringarray"},
                "not_length": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "length",
                "not_equals",
                "not_contains",
                "not_length",
                "is_null",
            ],
        },
        "stringchoice": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "equals": {"type": "stringchoice"},
                "is_null": {"type": "isnull"},
                "not_equals": {"type": "stringchoice"},
            },
            "sortedLookups": ["equals", "not_equals", "is_null"],
        },
        "stringchoicearray": {
            "defaultLookup": "equals",
            "defaultValue": None,
            "lookups": {
                "contains": {"type": "stringchoice"},
                "equals": {"type": "stringchoicearray"},
                "is_null": {"type": "isnull"},
                "length": {"type": "number"},
                "not_contains": {"type": "stringchoice"},
                "not_equals": {"type": "stringchoicearray"},
                "not_length": {"type": "number"},
            },
            "sortedLookups": [
                "equals",
                "contains",
                "length",
                "not_equals",
                "not_contains",
                "not_length",
                "is_null",
            ],
        },
        "unknown": {
            "defaultLookup": "is_null",
            "defaultValue": None,
            "lookups": {"is_null": {"type": "isnull"}},
            "sortedLookups": ["is_null"],
        },
    },
}

snapshots["test_query_csv_pivot_permutations[----] key"] = [[""]]

snapshots["test_query_csv_pivot_permutations[---b] key"] = [
    ["ID count", "Size max"],
    ["0.0", ""],
]

snapshots["test_query_csv_pivot_permutations[--c-] key"] = [
    ["Created time month"],
    [""],
]

snapshots["test_query_csv_pivot_permutations[--cb] key"] = [
    ["Created time month"],
    [""],
]

snapshots["test_query_csv_pivot_permutations[-r--] key"] = [["Created time year"]]

snapshots["test_query_csv_pivot_permutations[-r-b] key"] = [
    ["Created time year", "ID count", "Size max"]
]

snapshots["test_query_csv_pivot_permutations[-rc-] key"] = [
    ["Created time month"],
    ["Created time year"],
]

snapshots["test_query_csv_pivot_permutations[-rcb] key"] = [
    ["Created time month"],
    ["Created time year"],
]

snapshots["test_query_csv_pivot_permutations[d---] key"] = [[""]]

snapshots["test_query_csv_pivot_permutations[d--b] key"] = [
    ["ID count", "Size max"],
    ["6.0", "6.0"],
]

snapshots["test_query_csv_pivot_permutations[d-c-] key"] = [
    ["Created time month", "January", "Feburary"],
    [""],
    [""],
]

snapshots["test_query_csv_pivot_permutations[d-cb] key"] = [
    ["Created time month", "January", "", "Feburary", ""],
    ["", "ID count", "Size max", "ID count", "Size max"],
    ["", "4.0", "6.0", "2.0", "3.0"],
]

snapshots["test_query_csv_pivot_permutations[dr--] key"] = [
    ["Created time year"],
    ["2020.0"],
    ["2021.0"],
]

snapshots["test_query_csv_pivot_permutations[dr-b] key"] = [
    ["Created time year", "ID count", "Size max"],
    ["2020.0", "3.0", "3.0"],
    ["2021.0", "3.0", "6.0"],
]

snapshots["test_query_csv_pivot_permutations[drc-] key"] = [
    ["Created time month", "January", "Feburary"],
    ["Created time year"],
    ["2020.0"],
    ["2021.0"],
]

snapshots["test_query_csv_pivot_permutations[drcb] key"] = [
    ["Created time month", "January", "", "Feburary", ""],
    ["Created time year", "ID count", "Size max", "ID count", "Size max"],
    ["2020.0", "1.0", "1.0", "2.0", "3.0"],
    ["2021.0", "3.0", "6.0", "", ""],
]

snapshots["test_query_json data"] = {
    "body": [[{}, {}]],
    "cols": [{}],
    "fields": [
        {"pathStr": "size", "pivoted": False, "priority": 0, "sort": "dsc"},
        {"pathStr": "name", "pivoted": False, "priority": 1, "sort": "asc"},
        {"pathStr": "size_unit", "pivoted": False, "priority": None, "sort": None},
    ],
    "filterErrors": [None, None],
    "filters": [
        {"lookup": "lt", "pathStr": "size", "value": "2"},
        {"lookup": "gt", "pathStr": "id", "value": "0"},
    ],
    "formatHints": {
        "name": {},
        "size": {
            "highCutOff": 10000000000.0,
            "lowCutOff": 0.0001,
            "maximumFractionDigits": 0,
            "minimumFractionDigits": 0,
            "significantFigures": 3,
        },
        "size_unit": {},
    },
    "length": 2,
    "limit": 1000,
    "model": "core.Product",
    "rows": [
        {"name": "a", "size": 1.0, "size_unit": "g"},
        {"name": "b", "size": 1.0, "size_unit": "g"},
    ],
}

snapshots["test_query_json_pivot data"] = {
    "body": [
        [{"id__count": 1.0, "size__max": 1.0}, {"id__count": 3.0, "size__max": 6.0}],
        [{"id__count": 2.0, "size__max": 3.0}, None],
    ],
    "cols": [{"created_time__month": "January"}, {"created_time__month": "Feburary"}],
    "fields": [
        {
            "pathStr": "created_time__year",
            "pivoted": False,
            "priority": 0,
            "sort": "asc",
        },
        {
            "pathStr": "created_time__month",
            "pivoted": True,
            "priority": 1,
            "sort": "asc",
        },
        {"pathStr": "id__count", "pivoted": False, "priority": None, "sort": None},
        {"pathStr": "size__max", "pivoted": False, "priority": None, "sort": None},
    ],
    "filterErrors": [],
    "filters": [],
    "formatHints": {
        "created_time__month": {},
        "created_time__year": {
            "highCutOff": 10000000000.0,
            "lowCutOff": 0.0001,
            "maximumFractionDigits": 0,
            "minimumFractionDigits": 0,
            "significantFigures": 3,
            "useGrouping": False,
        },
        "id__count": {
            "highCutOff": 10000000000.0,
            "lowCutOff": 0.0001,
            "maximumFractionDigits": 0,
            "minimumFractionDigits": 0,
            "significantFigures": 3,
        },
        "size__max": {
            "highCutOff": 10000000000.0,
            "lowCutOff": 0.0001,
            "maximumFractionDigits": 0,
            "minimumFractionDigits": 0,
            "significantFigures": 3,
        },
    },
    "length": 3,
    "limit": 1000,
    "model": "core.Product",
    "rows": [{"created_time__year": 2020.0}, {"created_time__year": 2021.0}],
}



import pytest

from data_browser.models import View, global_data


@pytest.fixture
def view():
    return View(model_name="app.model", fields="+fa,-fd,fn", query="bob__equals=fred")


@pytest.fixture
def global_request(rf):
    request = rf.get("/")
    global_data.request = request
    yield request
    global_data.request = None


def test_str(view):
    view.name = "bob"
    assert str(view) == "app.model view: bob"


def test_public_link(view, global_request, settings):
    assert view.public_link() == "N/A"
    view.public = True
    expected = f"http://testserver/data_browser/view/{view.public_slug}.csv"
    assert view.public_link() == expected
    settings.DATA_BROWSER_ALLOW_PUBLIC = False
    assert view.public_link() == "Public Views are disabled in Django settings."


def test_google_sheets_formula(view, global_request, settings):
    assert view.google_sheets_formula() == "N/A"
    view.public = True
    expected = (
        f'=importdata("http://testserver/data_browser/view/{view.public_slug}.csv")'
    )
    assert view.google_sheets_formula() == expected
    settings.DATA_BROWSER_ALLOW_PUBLIC = False
    assert (
        view.google_sheets_formula() == "Public Views are disabled in Django settings."
    )



from urllib.parse import parse_qsl, urlencode

from django.contrib.auth import get_user_model
from django.test import RequestFactory
from django.urls import reverse

from data_browser.orm_admin import get_models
from data_browser.types import (
    IsNullType,
    NumberChoiceArrayType,
    NumberChoiceType,
    StringChoiceArrayType,
    StringChoiceType,
)


def _fix_filter(models, field, parts, lookup, value):
    if lookup == "is_null":
        value = {"true": "IsNull", "false": "NotNull"}.get(value.lower(), value)
    elif field.type_ == IsNullType and lookup == "equals":
        value = {"true": "IsNull", "false": "NotNull"}.get(value.lower(), value)
    elif field.type_ in [StringChoiceType, NumberChoiceType]:
        if lookup in ["equals", "not_equals"]:
            value, err = field.type_.raw_type.parse(lookup, value, None)
            choices = dict(field.choices)
            if err is None and value in choices:
                value = choices[value]
            else:
                parts.append("raw")
        elif lookup == "is_null":
            assert False  # should have been caught above
        else:
            parts.append("raw")
    elif field.type_ in [
        StringChoiceArrayType,
        NumberChoiceArrayType,
    ]:  # pragma: postgres
        if lookup in ["contains", "not_contains"]:
            value, err = field.type_.raw_type.parse(lookup, value, None)
            choices = dict(field.choices)
            if err is None and value in choices:
                value = choices[value]
            else:
                parts.append("raw")
    else:
        pass

    return parts, lookup, value


def forwards_0009(View):
    User = get_user_model()

    request = RequestFactory().get(reverse("admin:index"))
    request.user = User(is_superuser=True)
    models = get_models(request)

    for view in View.objects.all():
        filters = []
        for key, value in parse_qsl(view.query):
            *parts, lookup = key.split("__")

            model_name = view.model_name
            for part in parts:
                model = models[model_name]
                if part not in model.fields:
                    break
                field = model.fields[part]
                model_name = field.rel_name
            else:
                parts, lookup, value = _fix_filter(models, field, parts, lookup, value)

            key = "__".join(parts + [lookup])
            filters.append((key, value))
        view.query = urlencode(filters)
        view.save()



from django.db.models import Q

from .types import IsNullType, StringType


def get_django_filter(field_type, path_str, lookup, filter_value):
    # because on JsonFields Q(field=None) != Q(field__isnull=True)
    if lookup == "is_null":
        if filter_value is True:
            return Q(**{path_str: None}) | Q(**{f"{path_str}__isnull": True})
        elif filter_value is False:
            return ~Q(**{path_str: None})
        else:
            assert False

    if field_type == IsNullType:
        assert lookup == "equals", lookup
        if filter_value is True:
            return Q(**{path_str: None}) | Q(**{path_str: True})
        elif filter_value is False:
            return Q(**{path_str: False})
        else:
            assert False

    if lookup == "field_equals":
        lookup, filter_value = filter_value
    elif field_type == StringType:
        lookup = {
            "equals": "iexact",
            "regex": "iregex",
            "contains": "icontains",
            "starts_with": "istartswith",
            "ends_with": "iendswith",
            "is_null": "isnull",
        }[lookup]
    elif lookup == "contains":  # pragma: postgres
        filter_value = [filter_value]
    else:
        lookup = {
            "equals": "exact",
            "is_null": "isnull",
            "gt": "gt",
            "gte": "gte",
            "lt": "lt",
            "lte": "lte",
            "contains": "contains",
            "length": "len",
            "has_key": "has_key",
        }[lookup]

    return Q(**{f"{path_str}__{lookup}": filter_value})
